<?php
/**
 * ShopVoteModuleCenterModuleController.inc.php 2022-07-13
 *
 * @category  ShopVote
 * @package   shopvote/gambio_gx3
 * @link      https://www.shopvote.de
 *
 * @author    Valentin Fischer @ icue medienproduktion GmbH & Co. KG <contact@icue-medien.de>
 * @copyright ShopVote.de - Dirk Stuwe
 * @license   GPL-3.0
 * @license   https://opensource.org/licenses/GPL-3.0
 *
 * @author    Gambio GmbH
 * @version   2022-07-13
 */

use Gambio\CookieConsentPanel\Services\Purposes\Entities\Purpose;
use Gambio\CookieConsentPanel\Services\Purposes\PurposeReaderService;


class ShopVoteModuleCenterModuleController extends AbstractModuleCenterModuleController
{
    
    public function actionDefault()
    {
        $db       = StaticGXCoreLoader::getDatabaseQueryBuilder();
        /** @var \ShopVoteModuleCenterModuleInstaller $shopVoteModuleCenterModuleInstaller */
        $shopVoteModuleCenterModuleInstaller = MainFactory::create('ShopVoteModuleCenterModuleInstaller', $db);
        $shopVoteModuleCenterModuleInstaller->prepareDatabase();
        $title    = new NonEmptyStringType($this->pageTitle);
        $template = $this->getTemplateFile('shopvote_configuration.html');
        
        $shopvote = $db->limit(1)->order_by('`id`  DESC')->get('shopvote')->row_array();
        /*
        $reviewsHtml   = empty($shopvote['reviews_html']) ? '' : $shopvote['reviews_html'];
        $reviewsScript = empty($shopvote['reviews_script']) ? '' : $shopvote['reviews_script'];
        */
        $easyReviewsEnabled    = (bool)gm_get_conf('SHOPVOTE_EASYREVIEWS_ENABLED');
        $easyReviewsEnabled    = $easyReviewsEnabled || !empty($shopvote['reviews_script']);
        $productReviewsEnabled = (bool)gm_get_conf('SHOPVOTE_PRODUCTREVIEWS_ENABLED');
        $starsScript           = empty($shopvote['stars_script']) ? '' : $shopvote['stars_script'];
        $shopId                = (string)gm_get_conf('SHOPVOTE_SHOP_ID');
        $apiKey                = (string)gm_get_conf('SHOPVOTE_API_KEY');
        
        $numReviewsRow = $db->query('SELECT COUNT(*) AS num_reviews FROM `shopvote_reviews`')->row_array();
        $lastUpdateTimestamp = (int)gm_get_conf('SHOPVOTE_LAST_UPDATE_TIMESTAMP');
        if ($lastUpdateTimestamp === 0) {
            $lastUpdate = 'never';
        } else {
            $lastUpdateDateTime = new DateTime('@' . $lastUpdateTimestamp);
            $lastUpdateDateTime->setTimezone(new DateTimeZone('Europe/Berlin'));
            $lastUpdate = $lastUpdateDateTime->format('Y-m-d H:i:s');
        }

        $urlLanguageCode = '';
        if (gm_get_conf('USE_SEO_BOOST_LANGUAGE_CODE') === 'true') {
            $urlLanguageCode = defined('DEFAULT_LANGUAGE') ? DEFAULT_LANGUAGE : 'de';
            $urlLanguageCode .= '/';
        }
        $checkoutSuccessUrl = xtc_catalog_href_link($urlLanguageCode . 'checkout_success.php',
                                                    '',
                                                    'SSL');
    
        $formData = [
            'pageToken'               => $_SESSION['coo_page_token']->generate_token(),
            'actionSaveConfiguration' => xtc_href_link('admin.php', 'do=ShopVoteModuleCenterModule/SaveConfiguration'),
            'actionUpdateCache'       => xtc_href_link('admin.php', 'do=ShopVoteModuleCenterModule/UpdateCache'),
            'checkoutSuccessUrl'      => $checkoutSuccessUrl,
            'easyReviewsEnabled'      => $easyReviewsEnabled,
            'productReviewsEnabled'   => $productReviewsEnabled,
            'starsScript'             => $starsScript,
            'shopId'                  => $shopId,
            'apiKey'                  => $apiKey,
            'numberOfReviews'         => $numReviewsRow['num_reviews'],
            'lastUpdate'              => $lastUpdate,
            'purposes'                => $this->getCookieConsentPurposes(),
            'cookieConsentPurposeId'  => (int)(string)(gm_get_conf('SHOPVOTE_COOKIE_CONSENT_ID') ?? '0'),
        ];
        $data = MainFactory::create('KeyValueCollection', $formData);
    
        return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data);
    }
    
    
    public function actionSaveConfiguration()
    {
        $this->_validatePageToken();
        
        $shopId = (int)trim(stripslashes(strip_tags((string)$this->_getPostData('shopId'))));
        if (!empty($shopId)) {
            gm_set_conf('SHOPVOTE_SHOP_ID', $shopId);
        } else {
            $GLOBALS['messageStack']->add_session(
                $this->languageTextManager->get_text('shopvote_configuration_shopid_syntax_error'),
                'error'
            );
            $shopId = gm_get_conf('SHOPVOTE_SHOP_ID');
        }
        $apiKey = trim(stripslashes(strip_tags((string)$this->_getPostData('apiKey'))));
        if (preg_match('/^[\da-zA-Z]{32}$/', $apiKey) === 1) {
            gm_set_conf('SHOPVOTE_API_KEY', $apiKey);
        } else {
            $GLOBALS['messageStack']->add_session(
                $this->languageTextManager->get_text('shopvote_configuration_apikey_syntax_error'),
                'error'
            );
            $apiKey = gm_get_conf('SHOPVOTE_API_KEY');
        }
        $easyReviewsEnabled = !empty($shopId) && (bool)$this->_getPostData('easyReviewsEnabled');
        gm_set_conf('SHOPVOTE_EASYREVIEWS_ENABLED', $easyReviewsEnabled ? '1' : '0');
        $productReviewsEnabled = !empty($shopId) && !empty($apiKey)
                                 && (bool)$this->_getPostData(
                'productReviewsEnabled'
            );
        gm_set_conf('SHOPVOTE_PRODUCTREVIEWS_ENABLED', $productReviewsEnabled ? '1' : '0');
        gm_set_conf('SHOPVOTE_COOKIE_CONSENT_ID', (string)(int)$this->_getPostData('cookieConsentPurposeId'));
        
        $reviewsHtml = $reviewsScript = '';
        $starsScript = stripslashes((string)$this->_getPostData('starsScript'));
        // trim() the snippet; copypasting them tends to include some whitespace garbage
        $starsScriptLines = explode("\n", $starsScript);
        array_walk($starsScriptLines, static function(&$line) {
            $line = trim($line);
        });
        $starsScript = implode("\n", $starsScriptLines);
        $db          = StaticGXCoreLoader::getDatabaseQueryBuilder();
        $shopvote    = $db->limit(1)->order_by('`id`  DESC')->get('shopvote')->row_array();
        $db->set('reviews_html', $reviewsHtml)
           ->set('reviews_script', $reviewsScript)
           ->set('stars_script', $starsScript)
           ->where('id', (int)$shopvote['id'])
           ->update('shopvote');
        
        $GLOBALS['messageStack']->add_session(
            $this->languageTextManager->get_text('shopvote_configuration_saved'),
            'info'
        );
        
        return MainFactory::create(
            'RedirectHttpControllerResponse',
            xtc_href_link('admin.php', 'do=ShopVoteModuleCenterModule')
        );
    }
    
    public function actionUpdateCache()
    {
        $this->_validatePageToken();
    
        $shopId = new NonEmptyStringType((string)gm_get_conf('SHOPVOTE_SHOP_ID'));
        $apiKey = new NonEmptyStringType((string)gm_get_conf('SHOPVOTE_API_KEY'));
    
        /** @var \ShopvoteProductReviewsFactory $reviewsFactory */
        $reviewsFactory = MainFactory::create('ShopvoteProductReviewsFactory', $shopId, $apiKey);
        if ($this->_getPostData('cmd') === 'clear_cache') {
            $reviewsFactory->clearCache();
            $GLOBALS['messageStack']->add_session(
                $this->languageTextManager->get_text('SHOPVOTE_CACHE_CLEARED'),
                'info'
            );
        }
        if ($this->_getPostData('cmd') === 'update_cache') {
            $updateDays = (int)$this->_getPostData('update_days');
            if ($updateDays > 0) {
                $reviewsFactory->updateReviewsCache($updateDays);
            } else {
                $reviewsFactory->updateReviewsCache();
            }
            $GLOBALS['messageStack']->add_session(
                $this->languageTextManager->get_text('SHOPVOTE_REVIEWS_FETCHED'),
                'info'
            );
        }
        
        return MainFactory::create(
            'RedirectHttpControllerResponse',
            xtc_href_link('admin.php', 'do=ShopVoteModuleCenterModule')
        );
    }
    
    
    protected function _init()
    {
        $this->pageTitle = $this->languageTextManager->get_text('SHOPVOTE_TITLE');
    }
    
    protected function getCookieConsentPurposes(): array
    {
        $purposes = [];
        try {
            /** @var PurposeReaderService $purposeReaderService */
            $purposeReaderService = StaticGXCoreLoader::getService('PurposeReader');
            $allPurposes          = $purposeReaderService->allPurposes();
            /** @var Purpose $purpose */
            foreach ($allPurposes as $purpose) {
                $purposes[] = [
                    'id'   => $purpose->id()->value(),
                    'name' => $purpose->name()->value()[$_SESSION['languages_id']],
                ];
            }
        } catch (\Exception $e) {
        }
        
        return $purposes;
    }
}
