<?php
/* --------------------------------------------------------------
   ShopvoteLogger.inc.php 2019-11-27
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use Psr\Log\LoggerInterface;

class ShopvoteLogger implements LoggerInterface
{
    /** @var LogControl|null */
    protected $logControl;
    
    /** @var string */
    protected $logGroup;
    
    /** @var string */
    protected $logName;
    
    public function __construct()
    {
        $this->logControl = LogControl::get_instance();
        $this->logGroup = 'widgets';
        $this->logName = 'shopvote';
    }
    
    /**
     * Performs PSR-3 compliant context replacement.
     *
     * @param string $message
     * @param array  $context
     *
     * @return string
     */
    protected function replaceContext(string $message, array $context): string
    {
        $replace = [];
        foreach ($context as $placeHolder => $content) {
            if (!is_array($content) && (!is_object($content) || method_exists($content, '__toString'))) {
                $replace['{' . $placeHolder . '}'] = $content;
            }
        }
        
        return strtr($message, $replace);
    }
    
    /**
     * System is unusable.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function emergency($message, array $context = [])
    {
        $this->logControl->error($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Action must be taken immediately.
     *
     * Example: Entire website down, database unavailable, etc. This should
     * trigger the SMS alerts and wake you up.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function alert($message, array $context = [])
    {
        $this->logControl->error($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Critical conditions.
     *
     * Example: Application component unavailable, unexpected exception.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function critical($message, array $context = [])
    {
        $this->logControl->error($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Runtime errors that do not require immediate action but should typically
     * be logged and monitored.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function error($message, array $context = [])
    {
        $this->logControl->error($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function warning($message, array $context = [])
    {
        $this->logControl->warning($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Normal but significant events.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function notice($message, array $context = [])
    {
        $this->logControl->notice($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function info($message, array $context = [])
    {
        $this->logControl->notice($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Detailed debug information.
     *
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function debug($message, array $context = [])
    {
        $this->logControl->notice($this->replaceContext($message, $context), $this->logGroup, $this->logName . '-' . __FUNCTION__);
    }
    
    
    /**
     * Logs with an arbitrary level.
     *
     * @param mixed  $level
     * @param string $message
     * @param array  $context
     *
     * @return void
     */
    public function log($level, $message, array $context = [])
    {
        if (in_array($level, ['debug', 'info', 'warning', 'error', 'critical', 'alert', 'emergency'])) {
            return $this->$level($message, $context);
        }
    }
}
