<?php
/* --------------------------------------------------------------
   ShopvoteProductReviewsCollection.inc.php 2019-11-26
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ShopvoteProductReviewsCollection
{
    protected $productReviewsData;
    protected $productsId;
    
    public function __construct(IdType $productsId)
    {
        $this->productsId = $productsId;
    }
    
    public function getAge()
    {
        return time() - $this->productReviewsData['retrieved_at'];
    }
    
    
    /**
     * Uses response from single-product API endpoint for initialization.
     *
     * @param NonEmptyStringType $reviewsJsonResponse
     */
    public function initFromJsonData(NonEmptyStringType $reviewsJsonResponse)
    {
        $reviewsData = json_decode($reviewsJsonResponse->asString(), true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new InvalidArgumentException(json_last_error_msg());
        }
        if (!isset($reviewsData['retrieved_at'])) {
            $reviewsData['retrieved_at'] = time();
        }
        $this->productReviewsData = $reviewsData;
    }
    
    public function asJson()
    {
        return json_encode($this->productReviewsData);
    }

    public function initFromReviewsArray($reviews) {
        $this->productReviewsData = [
            'reviews' => $reviews,
        ];
    }
    
    public function getReviews()
    {
        if (isset($this->productReviewsData['reviews'])) {
            return $this->productReviewsData['reviews'];
        }
    
        return [];
    }
    
    public function getRatingCount()
    {
        $count = 0;
        if (array_key_exists('rating_count', $this->productReviewsData)) {
            $count = (int)$this->productReviewsData['rating_count'];
        } else {
            $count = count($this->productReviewsData['reviews']);
        }
        return $count;
    }
    
    public function getRatingValue()
    {
        $value = 0.0;
        if (array_key_exists('rating_value', $this->productReviewsData)) {
            $value = (float)$this->productReviewsData['rating_value'];
        } elseif (!empty($this->productReviewsData['reviews'])) {
            foreach ($this->productReviewsData['reviews'] as $review) {
                $value += $review['rating_value'];
            }
            $value /= count($this->productReviewsData['reviews']);
        }
        return $value;
    }
    
}
