<?php
/* --------------------------------------------------------------
   ShopVoteProductReviewsCheckoutSuccessExtender.inc.php 2022-07-13
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ShopVoteProductReviewsCheckoutSuccessExtender extends ShopVoteProductReviewsCheckoutSuccessExtender_parent
{
    public function proceed()
    {
        parent::proceed();
    
        $isInstalled = (int)gm_get_conf('MODULE_CENTER_SHOPVOTE_INSTALLED') === 1;
        if(!$isInstalled) {
            return;
        }
    
        $apiToken = (string)gm_get_conf('SHOPVOTE_API_KEY');

        $output = $this->getCustomerDataSnippet(
            $this->v_data_array['coo_order']->customer['email_address'],
            $this->v_data_array['orders_id']
        );
    
        $easyReviewEnabled = (bool)gm_get_conf('SHOPVOTE_EASYREVIEWS_ENABLED');
        $db       = StaticGXCoreLoader::getDatabaseQueryBuilder();
        if ($db->table_exists('shopvote')) {
            $shopvote = $db->limit(1)->order_by('`id` DESC')->get('shopvote')->row_array();
            $easyReviewEnabled = $easyReviewEnabled || !empty($shopvote['reviews_script']);
        }
        $productReviewsEnabled = (bool)gm_get_conf('SHOPVOTE_PRODUCTREVIEWS_ENABLED');
        if ($productReviewsEnabled) {
            $output .= $this->getProductsSnippet();
        }
        if ($easyReviewEnabled || $productReviewsEnabled) {
            $output .= $this->getReviewsV4Snippet($apiToken);
        }
    
        $this->html_output_array['SHOPVOTE_PRODUCT_REVIEWS'] = $output;
    }
    
    
    protected function getReviewsV4Snippet($apiToken)
    {
        $javascriptSnippet =
            '<script src="https://feedback.shopvote.de/srt-v4.min.js"></script>' .
            '<script>' .
                'var myToken = "' . $apiToken . '";' .
                'var mySrc = (\'https:\' === document.location.protocol ? \'https\' : \'http\');' .
                'loadSRT(myToken, mySrc);' .
            '</script>';
    
        $cookieConsentPurposeId = (int)gm_get_conf('SHOPVOTE_COOKIE_CONSENT_ID');
        if ($cookieConsentPurposeId > 0) {
            $javascriptSnippet = strtr($javascriptSnippet, [
                'src'     => 'data-src',
                'type'    => 'data-type',
                '<script' => '<script type="as-oil" data-type="application/javascript" data-managed="as-oil" '
                             . 'data-purposes="' . $cookieConsentPurposeId . '" ',
            ]);
        }
    
        return $javascriptSnippet;
    }
    
    
    protected function getCustomerDataSnippet($customerMail, $orderNumber)
    {
        $customerDataSnippet =
            '<div id="srt-customer-data" style="display:none;">' .
                '<span id="srt-customer-email">' . $customerMail . '</span>' .
                '<span id="srt-customer-reference">' . $orderNumber . '</span>' .
            '</div>';
        return $customerDataSnippet;
    }
    
    protected function getProductsSnippet()
    {
        $itemSnippets        = [];
        $languageCode        = MainFactory::create(
            'LanguageCode',
            MainFactory::create('StringType', $_SESSION['language_code'])
        );
        /** @var \ProductReadService $productReadService */
        $productReadService = StaticGXCoreLoader::getService('ProductRead');
        foreach ($this->v_data_array['coo_order']->products as $product) {
            $products_id = $product['id'];
            $seoBoost    = $GLOBALS['gmSEOBoost'];
            $product     = $productReadService->getProductById(MainFactory::create('IdType', $products_id));
            $productName = $product->getName($languageCode);
            if ($seoBoost->boost_products === true) {
                $productUrl = xtc_href_link(
                    $seoBoost->get_boosted_product_url(
                        $product->getProductId(),
                        $productName,
                        $_SESSION['languages_id']
                    )
                );
            } else {
                $productUrl = xtc_href_link('product_info.php', xtc_product_link($products_id, $productName));
            }
        
            $imageUrl = '';
            if ($product->getPrimaryImage()->isVisible() === true) {
                $imageUrl = xtc_href_link(
                    'images/product_images/popup_images/' . $product->getPrimaryImage()->getFilename(),
                    '',
                    'SSL',
                    false,
                    false,
                    false,
                    true,
                    true
                );
            }
        
            $brandName = $product->getAddonValues()->keyExists('brandName') ? $product->getAddonValues()->getValue(
                'brandName'
            ) : '';
        
            $itemSnippets[] = $this->getItemSnippet(
                $productUrl,
                $imageUrl,
                $productName,
                $product->getEan(),
                $product->getProductModel(),
                $brandName
            );
        }
    
        $checkoutProductsSnippet =
            '<div id="SHOPVOTECheckoutProducts" style="display: none;" translate="no">' . implode("\n", $itemSnippets) . '</div>';

        return $checkoutProductsSnippet;
    }

    protected function getItemSnippet($productUrl, $imageUrl, $productName, $gtin, $sku, $brandName)
    {
        $itemTemplate =
            '<span class="SVCheckoutProductItem">' .
                '<span class="sv-i-product-url">' . $productUrl . '</span>' .
                '<span class="sv-i-product-image-url">' . $imageUrl . '</span>' .
                '<span class="sv-i-product-name">' . $productName . '</span>' .
                '<span class="sv-i-product-gtin">' . $gtin . '</span>' .
                '<span class="sv-i-product-sku">' . $sku . '</span>' .
                '<span class="sv-i-product-brand">' . $brandName . '</span>' .
	        '</span>';
        return $itemTemplate;
    }
}
