<?php
/* --------------------------------------------------------------
   ShopVoteReviewsProduct.inc.php 2019-11-26
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ShopVoteReviewsProduct extends ShopVoteReviewsProduct_parent
{
    protected function isShopvoteReviewsEnabled()
    {
        static $isEnabled;
        if (isset($isEnabled)) {
            return $isEnabled;
        }
        $isInstalled = (int)gm_get_conf('MODULE_CENTER_SHOPVOTE_INSTALLED') === 1;
        $reviewsEnabled = (int)gm_get_conf('SHOPVOTE_PRODUCTREVIEWS_ENABLED') === 1;
        return $isInstalled && $reviewsEnabled;
    }
    
    public function getReviewsCount()
    {
        $count = parent::getReviewsCount();
        if ($this->isShopvoteReviewsEnabled() === true) {
            $shopvoteReviews = $this->getShopvoteReviews();
            $count += count($shopvoteReviews);
        }
        return $count;
    }

    public function getReviews($limit = false)
    {
        $reviews = parent::getReviews($limit);
        if ($this->isShopvoteReviewsEnabled() === true) {
            $svReviews = $this->retrieveShopvoteReviews();
            $reviews = array_merge($svReviews, $reviews);
            if ($limit === false) {
                $reviews = array_slice($reviews, 0, $limit);
            }
        }

        return $reviews;
    }
    
    public function getAggregateRatingData()
    {
        $shopAggregateRatingData = parent::getAggregateRatingData();
        if ($this->isShopvoteReviewsEnabled() === false) {
            return $shopAggregateRatingData;
        }
        $weightedShopRating = $shopAggregateRatingData['averageRating'] * $shopAggregateRatingData['count'];
    
        $reviewsCollection = $this->getShopvoteReviewsCollection();
        $weightedShopvoteRating = $reviewsCollection->getRatingValue() * $reviewsCollection->getRatingCount();
        
        $totalCount = $shopAggregateRatingData['count'] + $reviewsCollection->getRatingCount();
        if ($totalCount > 0) {
            $averageRating = ($weightedShopRating + $weightedShopvoteRating) / $totalCount;
        } else {
            $averageRating = 0;
        }

        $aggregateRatingData = [
            'count' => $totalCount,
            'averageRating' => $averageRating,
        ];
        
        return $aggregateRatingData;
    }
    
    protected function getShopvoteReviews()
    {
        static $shopvoteReviews = [];
        
        if (empty($shopvoteReviews)) {
            $shopvoteReviews = $this->retrieveShopvoteReviews();
        }
        return $shopvoteReviews;
    }

    protected function getShopvoteReviewsCollection()
    {
        $shopId = new NonEmptyStringType((string)gm_get_conf('SHOPVOTE_SHOP_ID'));
        $apiKey = new NonEmptyStringType((string)gm_get_conf('SHOPVOTE_API_KEY'));
    
        /** @var \ShopvoteProductReviewsFactory $reviewsFactory */
        $reviewsFactory = MainFactory::create('ShopvoteProductReviewsFactory', $shopId, $apiKey);
        $reviewsCollection = $reviewsFactory->getReviewsForProduct(new IdType((int)$this->pID));
        return $reviewsCollection;
    }
    
    protected function retrieveShopvoteReviews()
    {
        $convertedReviews = [];
        $reviewsCollection = $this->getShopvoteReviewsCollection();
        
        foreach ($reviewsCollection->getReviews() as $svReview) {
            $stars = (int)$svReview['rating_value'];
            $text = $svReview['review_text'] ?: strip_tags($svReview['text']);
            
            $convertedReviews[] = [
                'AUTHOR'       => $svReview['author'],
                'DATE'         => xtc_date_short($svReview['created']),
                'RATING'       => xtc_image(
                    StaticGXCoreLoader::getThemeControl()->getThemeImagePath() . 'stars_' . $stars . '.gif',
                    sprintf(BOX_REVIEWS_TEXT_OF_5_STARS, $stars)
                ),
                'TEXT'         => $text,
                'LINK'         => '',
                'DATE_CLEAN'   => date('Y-m-d', strtotime($svReview['created'])),
                'RATING_CLEAN' => $stars,
            ];
        }
        
        return $convertedReviews;
    }
}
